/*  OneWin VideoWin Benchmark Copyright (C) Roy Longbottom 2004
 *
 *  OneWin.exe runs the benchmark either stand-alone at existing screen
 *  settings or varying settings via VideoWin.exe. Results are saved
 *  in file VideoLog.txt. The following tests are run:
 *
 *  1 - BitBlt copy from sytem memory, usually cached in video RAM
 *  2 - Drawing small Ellipses
 *  3 - Recangle Fill
 *  4 - Setpixel
 *  5 - Drawing short lines
 *  6 - BitBlt copy screen to screen
 *  7 - Drawing 10000 Lines full screen
 *  8 - Drawing 1000 Ellipses full screen
 *
 *  Modules
 *
 *  config.cpp - via WhatConfig() identifies CPU type/speed, memory
 *               size, OS version, also running times via getStartTime()
 *               and getRunSecs().
 *  OneWin.rc -  Only icon 
 *  OneWin.h and OneWin.ico
 *
 *  OneWin.cpp
 *
 *  Particularly with Windows XP, on running a benchmark that displays
 *  on-going results, the window for the latter can be covered by
 *  another window. To overcome this, all tests are run via the WinMain 
 *  message processing loop using a runit variable, with a value same
 *  as above the test numbers:
 *
 *  Functions
 *
 *  WinMain() - usual Windows functions, runs setUp() and runTests()
 *  InitApp() - usual Windows functions, processes command line,
 *            allocates variable values
 *  WindowProc() - usual Windows functions for msgs, not used
 *
 *  setUp() - Opens log file, creates pattern in hdcDest for Test 1
 *            and another in hdcDest3 for general display then
 *            executes EndRoutine()
 *  EndRoutine() - Clears the screen, draws 6 picture frames for tests
 *            1 to 6, adds text and reproduces images in frames from
 *            tests run so far.
 *  runTests() - Runs Tests 1 to 6 according to runit variable for
 *            loopSecs (default 5) then EndRoutine(). Next is Test 7
 *            followed by lastTest()
 *  lastTest() - Test 8
 *  saveData() - WinMain() runit = 9 to save results to log file
 *            follwed by CleanUpAndPostQuit()
 *  CleanUpAndPostQuit() - Delete objects and exit
 *  gettimedate() - date and clock time
 *
 *  Input Parameters From VideoWin or BAT file
 *
 *  RUN
 *  RUNSECS - for each test
 *  Logfile save default all 
 *  SAVEHR  - save headings and results
 *  SAVER   - save results only
 *  SAVERE  - save results and end information
 *  
 */
 
#define NAME "WinTest" 
#define TITLE " Roy Longbottom's Windows Function Timing Test "
#define INITGUID

#include <windows.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <stdio.h>
#include <math.h>
#include <malloc.h>

#include "OneWin.h"


//  ************** PROTOPTYPES *****************

static BOOL InitApp(HINSTANCE, LPSTR, int);
long FAR PASCAL WindowProc(HWND, UINT, WPARAM, LPARAM);                  
static void CleanUpAndPostQuit(void);
static BOOL saveData();
static void setUp(HWND hwnd);
static void runTests(HDC hdc);
static BOOL EndRoutine(HDC hdc);
BOOL lastTest(HDC hdc);
void gettimedate();

char version[20] = "Version 3.2";
HANDLE    MyInstance;
HWND   mainHWND;
SYSTEMTIME sTime;
#define NUMENTRIES 256
PALETTEENTRY palentry[NUMENTRIES];
LOGPALETTE *pallog;
HPALETTE    hpal, hPalPrevious;
HBRUSH hbr;
HPEN hpen;
HGDIOBJ hpenOld;
HBITMAP hbmpNew;
HBITMAP hbmpNew2;
HBITMAP hbmpNew3;
BITMAP  bmp;
BITMAP  bmp2;
BITMAP  bmp3;
HDC  hdcDest;
HDC  hdcDest2;
HDC  hdcDest3;
FILE *outfile;


char    timedate[40];
char    commandLine[256] = "";
char    mbuff[256];
char    logfile[100] = "VideoLog.txt";
char    colordata[20];
char    screendata[50];
char    passtext[100];
char    test1res[50];
char    test2res[50];
char    test3res[50];
char    test4res[50];
char    test5res[50];
char    test6res[50];
char    test7res[50];

BOOL    autoRun = FALSE;
BOOL    bQuit = FALSE;
BOOL    saveHeadings = TRUE;
BOOL    saveResults = TRUE;
BOOL    saveEnd = TRUE;
BOOL    escPressed = FALSE;

int    xpixels;
int    ypixels;
int    bpixels;
int    ncolors;
int    xm;
int    ym;
int    whatsize;
int    randcol[512];
int    randWide[10000];
int    randHigh[10000];
int    incp, incq, incr, incs;
int    runit = 0;
int    param[10];
int    count[10];
int    keyPress;
int    looptime = 5;
int    thissize;
int    x[20];
int    y[20];
int    repeat7;

double  speed[10];
double  pixels[10];
double  thistime;
double  loopSecs = 5;    


DWORD gotPixel;


RECT   rectA, rect0, rect1, rect2, rect3, rect4, rect5, rect6;
RECT recwin;

// WinMain - initialization, message loop

int PASCAL WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                     LPSTR cmdline, int cmdshow )
{
    MSG         msg;
    HDC         hdc;
    
    /* previous instances do not exist in Win32 */
    if (hPrevInstance)
        return 0;
    
    cmdline = cmdline;


    if (!InitApp(hInstance, cmdline, cmdshow))
         return FALSE;

    gettimedate();
    
    if (!bQuit)
    {
        gotPixel = 0;
        runit = 1;
        setUp(mainHWND);
    }  

    while(!bQuit)
    {
        
        if( PeekMessage( &msg, NULL, 0, 0, PM_NOREMOVE ) )
        {
            
           if (msg.message == WM_QUIT)
           {
                CleanUpAndPostQuit();
                break;
           }
   
           if( !GetMessage( &msg, NULL, 0, 0 ) ) return msg.wParam;
           TranslateMessage(&msg);
           DispatchMessage(&msg);
        }
        else if (runit < 9)
        {
            hdc = GetDC(mainHWND);
            ShowCursor(FALSE);
            runTests(hdc);
            ShowCursor(TRUE);
            ReleaseDC(mainHWND, hdc);
            runit = runit + 1;
            if (runit < 7) EndRoutine(hdc);
        }
        else if (runit > 8)
        {
           saveData();
           CleanUpAndPostQuit();
        }
        else
        {
           // make sure we go to sleep if we have
           // nothing else to do
           WaitMessage();
        }
    }
    return msg.wParam;
    
} // WinMain

 
//               Initialization and object creation
// InitApp
// Creates window and initializes all objects neccessary to begin
// Rendering

static BOOL
InitApp(HINSTANCE hInstance, LPSTR cmdline, int cmdshow)
{
    HWND win;
    HDC hdc;
    WNDCLASS wc;
    int i, j;
    int x1, x2, y1, y2;
    
   
    // set up and registers the window class
    
    wc.style = CS_OWNDC; // | CS_HREDRAW | CS_VREDRAW; // CS_OWNDC;
    wc.lpfnWndProc = WindowProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = sizeof(DWORD);
    wc.hInstance = hInstance;
    wc.hIcon = LoadIcon(hInstance, "GenericIcon");
    wc.hCursor = LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground = (HBRUSH)GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName =  NULL; // "GenericMenu";
    wc.lpszClassName = NAME;
    if (!RegisterClass(&wc))
        return FALSE;
    
    // Initialize the global variables and allow the sample code
    // to override some of these default settings.
   
    // Create the window

    win =
        CreateWindow
        ( 
            NAME,                          // class
            TITLE,                         // caption 
            WS_OVERLAPPED | WS_CAPTION | WS_SYSMENU | WS_THICKFRAME | WS_MAXIMIZE,
            0,                             // init. x pos
            0,                             // init. y pos 
            GetSystemMetrics(SM_CXSCREEN), // init. x size
            GetSystemMetrics(SM_CYSCREEN), // init. y size
            NULL,                          // parent window
            NULL,                          // menu handle
            hInstance,                     // program handle
            NULL                           // create parms
        );
    if (!win)
        return FALSE;

    mainHWND = win;

    strcpy(commandLine, strupr(cmdline));    

    // command line scan

    if (strlen(commandLine))
    {
       for (j=0; j<lstrlen(commandLine); j++)
       {
          if (strnicmp(&commandLine[j],  "RUN", 3) == 0)
          {
              autoRun = TRUE;
          }
          if (strnicmp(&commandLine[j],  "RUNSECS ", 8) == 0)
          {
              sscanf(&commandLine[j+8], "%d", &looptime);
              if (looptime < 1) looptime = 1;
              if (looptime < 5)
              {
                  loopSecs = (double) looptime;
              }
              if (looptime > 900) looptime = 900;
              autoRun = TRUE;
          }
          if (strnicmp(&commandLine[j],  "SAVEHR", 6) == 0)
          {
              saveHeadings = TRUE;
              saveResults = TRUE;
              saveEnd = FALSE;
              autoRun = TRUE;
          }
          if (strnicmp(&commandLine[j],  "SAVER", 5) == 0)
          {
              saveHeadings = FALSE;
              saveResults = TRUE;
              saveEnd = FALSE;
              autoRun = TRUE;
          }
          if (strnicmp(&commandLine[j],  "SAVERE", 6) == 0)
          {
              saveHeadings = FALSE;
              saveResults = TRUE;
              saveEnd = TRUE;
              autoRun = TRUE;
          }
       }
    }
    
// ******************** Get Screen & Parameters ****************

    hdc = GetDC(NULL);

    xpixels = GetDeviceCaps(hdc, HORZRES);      
    ypixels = GetDeviceCaps(hdc, VERTRES);
    bpixels = GetDeviceCaps(hdc, BITSPIXEL);
    ym = ypixels / 8;
    xm = xpixels / 20;
    
    sprintf(screendata, "%d x %d, %d bits/pixel            ", xpixels, ypixels, bpixels);
    whatsize = 30;

    ReleaseDC(NULL, hdc); 
        
// *************** Random Numbers *********************

    srand(123);
    for (i=0; i<512; i++)
    {
         randcol[i] = (int)((float)rand() / 32768.0 * 220) + 1; 
    }
    for (i=0; i<10000; i++)
    {
        randWide[i] = (int)((float)rand() / 32768.0 * xpixels);
        randHigh[i] = (int)((float)rand() / 32768.0 * (ypixels-10))+10;
    }
    x1 = xm + 14;
    y1 = ym + 14;
    x2 = x1 + xpixels / 4 - 28;
    y2 = y1 + ypixels / 4 - 28;
    SetRect(&rect0, x1, y1, x2, y2);

    incs = (x2 - x1 + 1) / 2;
    
    y2 = ypixels - ym - 14;
    y1 = y2 - ypixels / 4 + 28;
    SetRect(&rect1, x1, y1, x2, y2);

    x2 = xpixels - xm - 14;
    x1 = x2 - xpixels / 4 + 28;
    y1 = ym + 14;
    y2 = y1 + ypixels / 4 - 28;
    SetRect(&rect2, x1, y1, x2, y2);

    y2 = ypixels - ym - 14;
    y1 = y2 - ypixels / 4 + 28;
    SetRect(&rect3, x1, y1, x2, y2);
    
    incp = (ypixels / 4 - 16) / 8;
    incq = 10;
    
    x1 = xpixels / 2 - incp * incq / 2 + 2;
    x2 = xpixels / 2 + incp * incq / 2 - 2;
    y2 = ypixels - ym - 14;
    y1 = y2 - incp * 8 + 4;
    SetRect(&rect4, x1-1, y1-1, x2+1, y2+1);

    incr = x1;
    if (xpixels < 800) x2 = x2 + 30;
    if (bpixels > 7)
     {
         x2 = x2 + 30;
         incr = incr - 30;
     }
    SetRect(&rectA, incr, ym / 2, x2, ym - 10);

    y1 = ym + 14;
    y2 = y1 + ypixels / 4 - 28;
    x1 = xpixels / 2 - incs;
    x2 = xpixels / 2 + incs;
    SetRect(&rect5, x1, y1, x2, y2);

    
    x1 = xm + 14;
    x2 = xpixels - xm - 14;    
    y1 = ym + ypixels / 4 + ym / 2;
    y2 = ypixels - ym - ypixels / 4 - ym / 2 - 14;
    SetRect(&rect6, x1, y1, x2, y2);

    // Display the window
     
    ShowWindow(win, cmdshow);   
    UpdateWindow(win);

    if (!autoRun)
    {
        sprintf(mbuff, "Command: %s\n"
                      "Test at %d x %d x %d bits\n\n"
                      "Click Cancel then VideoWin icon to\n"
                      "test at multiple screen settings\n\n" 
                      "At end, results should be\n"
                      "in file %s\n\n"
                      "Press OK to run", 
                   commandLine, xpixels, ypixels, bpixels, logfile);
       if (MessageBox(NULL, mbuff, "VideoWin Windows Speed Tests",
                    MB_ICONINFORMATION | MB_OKCANCEL) == IDCANCEL)
       {
           bQuit = TRUE;
       }
    }
    return TRUE;
}  // InitApp


// WindowProc - handle messages for the main application window
 

long FAR PASCAL WindowProc(HWND hwnd, UINT msg,
                            WPARAM wparam, LPARAM lparam)                  
{
        
    switch( msg )
    {
    case WM_ACTIVATEAPP:
        break;
        
    case WM_INITMENUPOPUP:
         break;

    case WM_CREATE:
        break;

    case WM_DESTROY:
        CleanUpAndPostQuit();
        break;

    case WM_PAINT:
        break;
           
    case WM_KEYDOWN:
        break;

    case WM_ACTIVATE:
        break;

    case WM_SIZE:
        return 0;

   case WM_PALETTECHANGED:
       break;

   case WM_QUERYNEWPALETTE: 
       break;

    case WM_COMMAND:
         break;
    } 
    return DefWindowProc( hwnd, msg, wparam, lparam );

} // WindowProc

    
void
CleanUpAndPostQuit(void)
{
     HDC  hdc;
     
     if (hdcDest2) DeleteDC(hdcDest2);
     if (hbmpNew2) DeleteObject(hbmpNew2);
     if (hdcDest3) DeleteDC(hdcDest3);
     if (hbmpNew3) DeleteObject(hbmpNew3);
     if (hdcDest) DeleteDC(hdcDest);
     if (hbmpNew) DeleteObject(hbmpNew);
     if (hpal)
     {
        hdc = GetDC(mainHWND);
        SelectPalette(hdc, hPalPrevious, FALSE);
        DeleteObject(hpal);
        ReleaseDC(mainHWND, hdc);
     }
     bQuit = TRUE;
}


void gettimedate()
{
    GetLocalTime(&sTime);
    sprintf(timedate, "%d/%d/%d %d:%d:%d \n",
                      sTime.wDay, sTime.wMonth, sTime.wYear,
                      sTime.wHour, sTime.wMinute, sTime.wSecond);
    return;
}

static void setUp(HWND hwnd)
{
    HDC hdc;
    int   h, i, j, k, pr;
    short red, blue, green;
    short r, g, b;
    int x1, x2, y1, y2;
    int xc1, yc1;

    outfile = fopen(logfile,"a+");  
    if (outfile == NULL)
    {
        MessageBox(mainHWND, "Cannot open results file", "Error", MB_ICONERROR | MB_OK);
        return;
    }
    if (saveHeadings)
    {
       fprintf(outfile, "\n    ***************************************************************************\n\n");   
       fprintf(outfile, "      Windows 9X/NT/2K/XP Video Test %s, %s\n", version, timedate);
       fprintf(outfile, "                  Copyright (C) 2001, Roy Longbottom\n\n");
       fprintf(outfile, "               .......... Millions of Pixels Per Second ........... ..Frames/Second..\n");
       fprintf(outfile, "  Resolution   BitBlt1  Ellipse Rectfill SetPixel DrawLine  BitBlt2 Lines10K Ellips1K\n\n"); 
       saveHeadings = FALSE;           
    }
    fflush (outfile);   
    
    hdc = GetDC(hwnd);
    GetClientRect(hwnd, &recwin);


   // *************** Set up Palette ***************************

    ncolors = NUMENTRIES; 
    
    pallog = (LOGPALETTE*) LocalAlloc(LPTR,
        sizeof(LOGPALETTE) + NUMENTRIES * sizeof(PALETTEENTRY));
    
    pallog->palNumEntries = NUMENTRIES;
    pallog->palVersion = 0x300;
    
    i = 0;
    for (red=0; red < 255; red += 36) 
     {
        for (green=0; green < 255; green += 36)
          {
             for (blue=0; blue < 256; blue += 85) 
              {    
                palentry[i].peRed =
                    pallog->palPalEntry[i].peRed = LOBYTE(red);   
                palentry[i].peGreen =
                    pallog->palPalEntry[i].peGreen = LOBYTE(green);
                palentry[i].peBlue =
                    pallog->palPalEntry[i].peBlue = LOBYTE(blue);
                palentry[i].peFlags =
                    pallog->palPalEntry[i].peFlags = PC_RESERVED;
                i = i + 1;
              }
          }
     }
    i = 0;
    for (red=0; red < 129; red += 128) 
    {
        for (green=0; green < 129; green += 128)
          {
             for (blue=0; blue < 129; blue += 128) 
              {
                r = red   / 128 * 127;
                g = green / 128 * 127;
                b = blue  / 128 * 127;
                if (i == 0)
                 {
                     r = 128;
                     b = 128;
                     g = 128;
                 }
                j=0;
                if (i==7) j = 64;
                palentry[i].peRed =
                    pallog->palPalEntry[i].peRed = LOBYTE(red+j);   
                palentry[i].peGreen =
                    pallog->palPalEntry[i].peGreen = LOBYTE(green+j);
                palentry[i].peBlue =
                    pallog->palPalEntry[i].peBlue = LOBYTE(blue+j);
                palentry[i].peFlags =
                    pallog->palPalEntry[i].peFlags = PC_RESERVED;
                palentry[i+8].peRed =
                    pallog->palPalEntry[i+8].peRed = LOBYTE(red+r);   
                palentry[i+8].peGreen =
                    pallog->palPalEntry[i+8].peGreen = LOBYTE(green+g);
                palentry[i+8].peBlue =
                    pallog->palPalEntry[i+8].peBlue = LOBYTE(blue+b);
                palentry[i+8].peFlags =
                    pallog->palPalEntry[i+8].peFlags = PC_RESERVED;
                i = i + 1;
              }
          }
    }
    
    hpal = CreatePalette(pallog);
    LocalFree((HLOCAL) pallog);

    hPalPrevious = SelectPalette(hdc, hpal, FALSE);
    pr = RealizePalette(hdc);

// *************** End Set up Palette ***************************

    // ********** Fill screen black *******
    
    hbr = CreateSolidBrush(PALETTEINDEX(0));  
    SelectObject(hdc, hbr);
    Rectangle(hdc, recwin.left, recwin.top, recwin.right, recwin.bottom);
    DeleteObject(hbr);
    
    // *********** Create Pattern For Test 1 ************

    x1 = xm + 14;
    y1 = ym;
    x2 = x1 + 1;
    y2 = y1 + 1;

    j = 0;
    h = 2;
    for (i=20; i < xpixels*2/11; i++)
    {              
          k = i+20;
          hpen = CreatePen(PS_SOLID, 8, PALETTEINDEX(j));
          hpenOld = SelectObject(hdc, hpen);
          Rectangle(hdc, x1+i-20+k*2, y1+i-20, x2+(i-20)*3/2+k*2, y2+(i-20)*3/2);
          Ellipse(hdc,   x1+i-20+k,   y1+i-20, x2+(i-20)*3/2+k,   y2+(i-20)*3/2);
          Rectangle(hdc, x1+i-18,     y1+i-20, x2+(i-18)*3/2,     y2+(i-20)*3/2);
          SelectObject(hdc, hpenOld);
          DeleteObject(hpen);
          j = j + h;
          if (j > 220)
          {
               j = 1;
               h = 1;
          }
          else if (j==16)
          {
               h = 3;
               j = 170;
          }    
    }
    x1 = xm;
    x2 = x1 + xpixels / 4;
    y1 = ym + xpixels/4 + 50;
    y2 = y1 + ypixels / 4;
    xc1 = x1 + 14;
    yc1 = y1 + 14;

    j = 12;
    for (i=0; i<j; i++)
    {
        hpen = CreatePen(PS_SOLID, 0, PALETTEINDEX(180+i*4));
        hpenOld = SelectObject(hdc, hpen);
        Rectangle(hdc, x1+i, y1+i, x2-i, y2-i);
        SelectObject(hdc, hpenOld);
        DeleteObject(hpen);
    }
    
    x1 = xm + 12;
    y1 = ym;
    x2 = x1 + xpixels / 4 - 28;
    y2 = y1 + ypixels / 4 - 28;

    BitBlt(hdc, xc1, yc1, (x2-x1), (y2-y1), hdc, x1, y1, SRCCOPY);


    hbmpNew = CreateDiscardableBitmap(hdc, x2-x1, y2-y1);
    GetObject(hbmpNew, sizeof(BITMAP), &bmp);
    hdcDest = CreateCompatibleDC(hdc);
    SelectObject(hdcDest, hbmpNew);

    BitBlt(hdcDest, 0, 0, (x2-x1), (y2-y1), hdc, x1, y1, SRCCOPY); // keep copy

    // **************** All colors middle ***********
    
    x1 = rect6.left;
    y1 = rect6.top;
    x2 = rect6.right;
    y2 = rect6.bottom;
    
    j = 0;
    for (i=x1; i<x2+1; i++)
    {
        hpen = CreatePen(PS_SOLID, 0, PALETTEINDEX(j));   
        hpenOld = SelectObject(hdc, hpen);
        Rectangle(hdc, i, y1, x2, y2);
        j = j + 1;
        if (j > ncolors) j = 0;
        SelectObject(hdc, hpenOld);
        DeleteObject(hpen);
    }

    hbmpNew3 = CreateDiscardableBitmap(hdc, x2-x1, y2-y1);
    GetObject(hbmpNew3, sizeof(BITMAP), &bmp2);
    hdcDest3 = CreateCompatibleDC(hdc);
    SelectObject(hdcDest3, hbmpNew3);

    BitBlt(hdcDest3, 0, 0, (x2-x1), (y2-y1), hdc, x1, y1, SRCCOPY); // keep copy

    for (i=0; i<10; i++)
    {
        speed[i] = 0;
        count[i] = 0;
    }

    keyPress = GetAsyncKeyState(VK_ESCAPE);

    param[0] = 1;
    param[1] = 25;
    param[2] = 3;
    param[3] = 4;
    param[4] = 50;
    param[5] = 5; 

    if (!EndRoutine(hdc)) return;

    ReleaseDC(hwnd, hdc); 

} // setUp


static void runTests(HDC hdc)
{
    int i, j, k, m, t;
    int x1, x2, x3, x4, y1, y2, y3, y4;
    int i1, i2;
    
   
    if (runit == 1)
    {
       // ************ Fill rectangles black *************
       
        hbr = CreateSolidBrush(PALETTEINDEX(0)); 
        SelectObject(hdc, hbr);
        Rectangle(hdc, rect0.left, rect0.top, rect0.right, rect0.bottom);
        Rectangle(hdc, rect1.left, rect1.top, rect1.right, rect1.bottom);
        Rectangle(hdc, rect2.left, rect2.top, rect2.right, rect2.bottom);
        Rectangle(hdc, rect3.left, rect3.top, rect3.right, rect3.bottom);
        Rectangle(hdc, rect4.left, rect4.top, rect4.right, rect4.bottom);   
        Rectangle(hdc, rect5.left, rect5.top, rect5.right, rect5.bottom);
        DeleteObject(hbr);

        // ***************** Test 1 Memory BitBlt *******************

        x1 = rect0.left;
        y1 = rect0.top;
        x2 = rect0.right;
        y2 = rect0.bottom;
       
        x3 = x2 - x1 - 1; 
        pixels[0] = (double)(x3+1) * 4 / 2.0  * (double) (y2-y1+1);

        sprintf (test1res, "BitBlt from memory                   ");
        sprintf (passtext, " Test 1                          ");
        thissize = 24;
        ExtTextOut(hdc, xm, ym / 2, ETO_CLIPPED, &recwin, passtext, thissize, (LPINT) NULL);
        thistime = 0;
        for (t=0; t<looptime; t=t+5)
        {
            thissize = 24;
            ExtTextOut(hdc, rect0.left-12, rect0.bottom+15, ETO_CLIPPED, &recwin, test1res, thissize, (LPINT) NULL);
            getStartTime();
            do
            {
              for (j=0; j<param[0]; j++)
              {
                   for (i=0; i < x3; i++)
                   {
                        BitBlt(hdc, rect0.left+i+1, rect0.top, (x3-i), (y2-y1), hdcDest, 0, 0, SRCCOPY);
                        BitBlt(hdc, rect1.left+i+1, rect1.top, (x3-i), (y2-y1), hdcDest, 0, 0, SRCCOPY);
                        BitBlt(hdc, rect2.left+i+1, rect2.top, (x3-i), (y2-y1), hdcDest, 0, 0, SRCCOPY);
                        BitBlt(hdc, rect3.left+i+1, rect3.top, (x3-i), (y2-y1), hdcDest, 0, 0, SRCCOPY);
                   }   
                   for (i=0; i < x3; i++)
                   {
                        BitBlt(hdc, rect0.left+x3-i, rect0.top, (1+i), (y2-y1), hdcDest, 0, 0, SRCCOPY);
                        BitBlt(hdc, rect1.left+x3-i, rect1.top, (1+i), (y2-y1), hdcDest, 0, 0, SRCCOPY);
                        BitBlt(hdc, rect2.left+x3-i, rect2.top, (1+i), (y2-y1), hdcDest, 0, 0, SRCCOPY);
                        BitBlt(hdc, rect3.left+x3-i, rect3.top, (1+i), (y2-y1), hdcDest, 0, 0, SRCCOPY);
                   }   
              }
              GdiFlush();
              SetPixel(hdc, 2, 1, RGB(0,0,0));
              gotPixel = gotPixel | GetPixel(hdc, 2, 1);
              getRunSecs();
              count[0] = count[0] + param[0];
              if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
              {
                  runit = 9;
                  escPressed = TRUE;
                  return;
              }            
            }   
            while (runSecs <  loopSecs);
            thistime = thistime + runSecs;
            if (!EndRoutine(hdc)) return;
        }
        speed[0] = pixels[0] * (double)count[0] * (double)(x3 * 2)/ thistime / 1000000;

        sprintf (test1res, "BitBlt %8.1f MP/sec       ", speed[0]);
    }
    else if (runit == 2)
    {
      // ***************** Test 2 Ellipse *****************

        x[0] = rect4.left - 1;
        y[0] = rect4.top - 1;
     
        for (j=1; j<incq; j++)
        {
             x[j] = x[j-1] + incp;
             y[j] = y[j-1] + incp;
        }
         
        sprintf (test2res, "Ellipse                          ");
        sprintf (passtext, " Test 2                          ");
        thissize = 24;
        ExtTextOut(hdc, xm, ym / 2, ETO_CLIPPED, &recwin, passtext, thissize, (LPINT) NULL);
        thistime = 0;
        for (t=0; t<looptime; t=t+5)
        {
            thissize = 24;
            ExtTextOut(hdc, rect4.left-12, rect4.top-30, ETO_CLIPPED, &recwin, test2res, thissize, (LPINT) NULL);
            getStartTime();    
            do
            {
              for (i=0; i < param[1]; i++)
              {       
                   for (j=0; j<incq; j++)   // incq = 10
                   {
                        for (k=0; k<8; k++)
                        {
                             hbr = CreateSolidBrush(PALETTEINDEX(randcol[i+k+j*8])); 
                             SelectObject(hdc, hbr);
                             Ellipse(hdc,  x[j], y[k], x[j] + incp, y[k] + incp);
                             DeleteObject(hbr);
                        }
                   }   
              }
              GdiFlush();
              SetPixel(hdc, 2, 1, RGB(0,0,0));
              gotPixel = gotPixel | GetPixel(hdc, 2, 1);
              getRunSecs();
              count[1] = count[1] + param[1];
              if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
              {
                  runit = 9;
                  escPressed = TRUE;
                  return;
              }            
            }   
            while (runSecs < loopSecs);
            thistime = thistime + runSecs;
            if (!EndRoutine(hdc)) return;
        }            
        pixels[1] = (double)(8 * incq) * (double)(incp * incp) / 4.0 * 3.142;
                                      // 2r * 2r / 4 = r^2
        speed[1] = pixels[1] * (double)count[1] / thistime / 1000000;        
        sprintf (test2res, "Ellipse %8.2f MP/sec      ", speed[1]);
    } 
    else if (runit == 3)
    {
        // ****************** Test 3 Fill *****************

        sprintf (test3res, "Fill rectangle                    ");
        sprintf (passtext, " Test 3                          ");
        thissize = 24;
        ExtTextOut(hdc, xm, ym / 2, ETO_CLIPPED, &recwin, passtext, thissize, (LPINT) NULL);
        thistime = 0;
        for (t=0; t<looptime; t=t+5)
        {
            thissize = 24;
            ExtTextOut(hdc, rect1.left-12, rect1.top-30, ETO_CLIPPED, &recwin, test3res, thissize, (LPINT) NULL);
            getStartTime();    
            do
            {
                for (j=0; j<param[2]; j++)
                {
                   for (i=0; i < ncolors; i=i+3)
                   {
                         hbr = CreateSolidBrush(PALETTEINDEX(randcol[i])); 
                         SelectObject(hdc, hbr);          
                         Rectangle(hdc, rect1.left, rect1.top, rect1.right, rect1.bottom);
                         Rectangle(hdc, rect2.left, rect2.top, rect2.right, rect2.bottom);
                         Rectangle(hdc, rect3.left, rect3.top, rect3.right, rect3.bottom);            
                         DeleteObject(hbr);                            
                   }   
                }
                GdiFlush();   
                SetPixel(hdc, 2, 1, RGB(0,0,0));
                gotPixel = gotPixel | GetPixel(hdc, 2, 1);
                getRunSecs();
                count[2] = count[2] + param[2];
                if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
                {
                  runit = 9;
                  escPressed = TRUE;
                  return;
                }            
            }   
            while (runSecs < loopSecs);
            thistime = thistime + runSecs;
            if (!EndRoutine(hdc)) return;
        }                            
        pixels[2] = (double)(rect1.right-rect1.left+1) * (double)(rect1.bottom-rect1.top+1);
        speed[2] = pixels[2] * (double)count[2] * (double)ncolors / thistime / 1000000;
        sprintf (test3res, "Fill %8.1f MP/sec          ", speed[2]);
    }
    else if (runit == 4)
    {
         // ****************** Test 4 Setpixel ****************

        hbr = CreateSolidBrush(PALETTEINDEX(14)); 
        SelectObject(hdc, hbr);          
        sprintf (test4res, "SetPixel                                  ");
        sprintf (passtext, " Test 4                          ");
        thissize = 24;
        ExtTextOut(hdc, xm, ym / 2, ETO_CLIPPED, &recwin, passtext, thissize, (LPINT) NULL);
        thistime = 0;
        for (t=0; t<looptime; t=t+5)
        {
              thissize = 25;
              ExtTextOut(hdc, rect2.left-12, rect2.bottom+15, ETO_CLIPPED, &recwin, test4res, thissize, (LPINT) NULL);                  
              getStartTime();    
              do
              {
                  m=0;
                  for (j=0; j<param[3]; j++)
                  {
                       Rectangle(hdc, rect2.left, rect2.top, rect2.right, rect2.bottom);
                       Rectangle(hdc, rect3.left, rect3.top, rect3.right, rect3.bottom);
                       for (i=rect2.left; i < rect2.right+1; i++)
                       {
                            for (k=rect2.top; k<rect2.bottom+1; k++)
                            {
                               SetPixel(hdc, i, k, PALETTEINDEX(randcol[m]));
                               m = m + 1;
                               if (m > 500) m = 0;                 
                            }
                            for (k=rect3.top; k<rect3.bottom+1; k++)
                            {
                               SetPixel(hdc, i, k, PALETTEINDEX(randcol[m]));
                               m = m + 1;
                               if (m > 500) m = 0;                 
                            }
                      }
                  }
                  GdiFlush();    
                  SetPixel(hdc, 2, 1, RGB(0,0,0));
                  gotPixel = gotPixel | GetPixel(hdc, 2, 1);
                  getRunSecs();
                  count[3] = count[3] + param[3];
                  if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
                  {
                     runit = 9;
                     escPressed = TRUE;
                     return;
                  }            
              }   
              while (runSecs < loopSecs);
              thistime = thistime + runSecs;
              if (!EndRoutine(hdc))
              {
                  DeleteObject(hbr);
                  return;
              }
        }                            
        pixels[3] = 2 * (double)(rect2.right-rect2.left+1) * (double)(rect2.bottom-rect2.top+1);   
        speed[3] = pixels[3] * (double)count[3]  / thistime / 1000000;
        sprintf (test4res, "SetPixel %8.3f MP/sec          ", speed[3]);

        DeleteObject(hbr);
          
        x1 = rect2.left;
        y1 = rect2.top;
        x2 = rect2.right;
        y2 = rect2.bottom;

        hbmpNew2 = CreateDiscardableBitmap(hdc, x2-x1, y2-y1);
        GetObject(hbmpNew2, sizeof(BITMAP), &bmp2);
        hdcDest2 = CreateCompatibleDC(hdc);
        SelectObject(hdcDest2, hbmpNew2);
        BitBlt(hdcDest2, 0, 0, (x2-x1), (y2-y1), hdc, x1, y1, SRCCOPY); // keep copy
    }
    else if (runit == 5)
    {
        // ****************** Test 5 Line to *****************
         
        x1 = rect3.left;
        x2 = rect3.right;
        y1 = rect3.top;
        y2 = rect3.bottom;        
        i1 = 8;
        if (xpixels >  800) i1 = 16;
        if (xpixels == 800) i1 = 12;
        i2 = i1/2;
        m = 1;
        pixels[4] = 0;                    
        sprintf (test5res, "LineTo                                  ");
        sprintf (passtext, " Test 5                          ");
        thissize = 24;
        ExtTextOut(hdc, xm, ym / 2, ETO_CLIPPED, &recwin, passtext, thissize, (LPINT) NULL);
        thistime = 0;
        for (t=0; t<looptime; t=t+5)
        {
              thissize = 24;
              ExtTextOut(hdc, rect3.left-12, rect3.top-30, ETO_CLIPPED, &recwin, test5res, thissize, (LPINT) NULL);         
              getStartTime();    
              do
              {
                  for (j=0; j<param[4]; j++)
                  {
                       x3 = x2 - x1 - i1;
                       x4 = x1 + i2;
                       y3 = y2 - y1 - i1;
                       y4 = y1 + i2;
                       MoveToEx(hdc, x4, y4, NULL);
                       hpen = CreatePen(PS_SOLID, 0, PALETTEINDEX(m));
                       hpenOld = SelectObject(hdc, hpen);
                       m = m +1;
                       if (m > 15) m = 1;
                       for (i=0; i < 9; i++)
                       {                  
                           LineTo(hdc, x4+x3, y4);
                           LineTo(hdc, x4+x3, y4+y3);
                           LineTo(hdc, x4,    y4+y3);
                           LineTo(hdc, x4,    y4+i2);
                           pixels[4] = pixels[4] + (double)(2 * x3) + (double)(2 * y3 - i2);
                           x3 = x3 - i1;
                           y3 = y3 - i1;
                           x4 = x4 + i2;
                           y4 = y4 + i2;                       
                       }
                       SelectObject(hdc, hpenOld);
                       DeleteObject(hpen);
                  }
                  GdiFlush();                  
                  SetPixel(hdc, 2, 1, RGB(0,0,0));
                  gotPixel = gotPixel | GetPixel(hdc, 2, 1);
                  getRunSecs();
                  count[4] = count[4] + param[4];
                  if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
                  {
                     runit = 9;
                     escPressed = TRUE;
                     return;
                  }            
              }   
              while (runSecs < loopSecs);
              thistime = thistime + runSecs;
              if (!EndRoutine(hdc)) return;
        }
        speed[4] = pixels[4]  / thistime / 1000000;                
        sprintf (test5res, "LineTo %8.2f MP/sec          ", speed[4]);
    }
    else if (runit == 6)
    {
        // ***************** BitBlt from screen ******************
     
        x1 = rect0.left;
        x2 = rect0.right;
        y1 = rect0.top;
        y2 = rect0.bottom;
        x3 = x2 - x1 - 1;

        sprintf (test6res, "BitBlt from screen                           ");

        pixels[5] = (double)(x3+1) * 4 / 2.0  * (double) (y2-y1+1);                   
        sprintf (passtext, " Test 6                          ");
        thissize = 24;
        ExtTextOut(hdc, xm, ym / 2, ETO_CLIPPED, &recwin, passtext, thissize, (LPINT) NULL);
        thistime = 0;
        for (t=0; t<looptime; t=t+5)
        {
            BitBlt(hdc, rect5.left, rect5.top, (x2-x1), (y2-y1), hdc, rect0.left, rect0.top, SRCCOPY);            
            thissize = 24;
            ExtTextOut(hdc, rect5.left-12, rect5.bottom+15, ETO_CLIPPED, &recwin, test6res, thissize, (LPINT) NULL);         
            getStartTime();    
            do
            {
               for (j=0; j<param[5]; j++)
               {
                   for (i=0; i < x3; i++)
                   {
                        BitBlt(hdc, rect0.left+i+1, rect0.top, (x3-i), (y2-y1), hdc, rect5.left, rect5.top, SRCCOPY);
                        BitBlt(hdc, rect1.left+i+1, rect1.top, (x3-i), (y2-y1), hdc, rect5.left, rect5.top, SRCCOPY);
                        BitBlt(hdc, rect2.left+i+1, rect2.top, (x3-i), (y2-y1), hdc, rect5.left, rect5.top, SRCCOPY);
                        BitBlt(hdc, rect3.left+i+1, rect3.top, (x3-i), (y2-y1), hdc, rect5.left, rect5.top, SRCCOPY);
                   }   
                   for (i=0; i < x3; i++)
                   {
                        BitBlt(hdc, rect0.left+x3-i, rect0.top, (1+i), (y2-y1), hdc, rect5.left, rect5.top, SRCCOPY);
                        BitBlt(hdc, rect1.left+x3-i, rect1.top, (1+i), (y2-y1), hdc, rect5.left, rect5.top, SRCCOPY);
                        BitBlt(hdc, rect2.left+x3-i, rect2.top, (1+i), (y2-y1), hdc, rect5.left, rect5.top, SRCCOPY);
                        BitBlt(hdc, rect3.left+x3-i, rect3.top, (1+i), (y2-y1), hdc, rect5.left, rect5.top, SRCCOPY);
                   }   
               }
               GdiFlush();
               SetPixel(hdc, 2, 1, RGB(0,0,0));
               gotPixel = gotPixel | GetPixel(hdc, 2, 1);
               getRunSecs();
               count[5] = count[5] + param[5];
               if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
               {
                 runit = 9;
                 escPressed = TRUE;
                 return;
               }            
            }   
            while (runSecs < loopSecs);
            thistime = thistime + runSecs;
            if (!EndRoutine(hdc)) return;
        }                        
        speed[5] = pixels[5] * (double)count[5] * (double)(x3 * 2) / thistime / 1000000;
        sprintf (test6res, "BitBlt %8.2f MP/sec          ", speed[5]);
        sprintf (test7res, "GetPixel 0=0X%8.8X               ", gotPixel);
    }    
    else if (runit == 7)
    {
        // ********** Fill screen white/grey *******
            
        hbr = CreateSolidBrush(PALETTEINDEX(0)); // 7  
        SelectObject(hdc, hbr);
        Rectangle(hdc, recwin.left, recwin.top, recwin.right, recwin.bottom);
        DeleteObject(hbr);

        // ************ 10000 Lines *****************************

        sprintf (passtext, " 10000 lines                        ");
        thissize = 24;
        ExtTextOut(hdc, xm, ym / 2, ETO_CLIPPED, &recwin, passtext, thissize, (LPINT) NULL);
        thistime = 0;
        count[6] = 0;
        MoveToEx(hdc, 1, 1, NULL);
        getStartTime();
        do
        {
           m = 0;
           k = 1;       
           for (i=0; i < 200; i++)
           {
                 hpen = CreatePen(PS_SOLID, 0, PALETTEINDEX(k)); // randcol[i]
                 hpenOld = SelectObject(hdc, hpen);
                 k = k + 1;
                 if (k > 220) k = 1;
                 for (j=0; j<50; j++)
                 {
                     LineTo(hdc, randWide[m], randHigh[m]);
                     m = m + 1;         
                 }
                 SelectObject(hdc, hpenOld);
                 DeleteObject(hpen);
           }
           count[6] = count[6] + 1;
           getRunSecs();
           speed[6] = (double)count[6] / runSecs;
           sprintf (passtext, " 10000 Lines %6.2f Frames Per Second                   ", speed[6]);
           thissize = 45;
           ExtTextOut(hdc, rect5.left, 1, ETO_CLIPPED, &recwin, passtext, thissize, (LPINT) NULL);
           if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
           {
               runit = 9;
               escPressed = TRUE;
               return;
           }        
        }
        while (runSecs <  (double)looptime);
        speed[6] = (double)count[6] / runSecs;
        sprintf (passtext, "%6.2f FPS", speed[6]);
    }
    else if (runit == 8)
    {
        // ********** Fill screen white/grey *******
            
        hbr = CreateSolidBrush(PALETTEINDEX(0)); // 7  
        SelectObject(hdc, hbr);
        Rectangle(hdc, recwin.left, recwin.top, recwin.right, recwin.bottom);
        DeleteObject(hbr);
        count[7] = 0;
        repeat7 = 10;
        k = 1;
        repeat7 = 1;
        getStartTime();
        if (!lastTest(hdc)) return;
        getRunSecs();
        if (runSecs < 1) repeat7 = (int)( 1 / runSecs);        
        k = 1;
        getStartTime();
        do
        {
            if (!lastTest(hdc)) return;
            count[7] = count[7] + repeat7;
            getRunSecs();
            speed[7] = (double)count[7] / runSecs;
            sprintf (passtext, " 1000 Ellipses %6.2f Frames Per Second                   ", speed[7]);
            thissize = 45;
            ExtTextOut(hdc, rect5.left, 1, ETO_CLIPPED, &recwin, passtext, thissize, (LPINT) NULL);
            if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
            {
                runit = 9;
                escPressed = TRUE;
                return;
            }        
        }
        while (runSecs <  (double)looptime);
    }

} // runTests

BOOL lastTest(HDC hdc)
{
    double dx, dy, dxi, dyi;
    int x1, x2, x3, y1, y2, y3;
    int i, j, k, r;

    k = 1;
    x1 = 0;
    x2 = (xpixels - 60) / 30;  // 21
    x3 = (xpixels - 60) / 30; 
    y1 = 0;
    y2 = (ypixels - 60) / 30;
    y3 = (ypixels - 60) / 30;
    dxi = (double)x3 / 25;
    dyi = (double)y3 / 25;
    for (i=0; i<20; i++)
    {
        dx = 0.0;
        dy = 0.0;
        for (j=0; j<25; j++)
        {
            dx = dx + dxi;
            dy = dy + dyi;
            for (r=0; r<repeat7; r++)
            {
                hbr = CreateSolidBrush(PALETTEINDEX(k)); 
                SelectObject(hdc, hbr);
                Ellipse(hdc, x1+(int)dx, y1+(int)dy, x2+(int)dx, y2+(int)dy);
                Ellipse(hdc, xpixels-x2-1-(int)dx, y1+(int)dy, xpixels-x1-(int)dx, y2+(int)dy);
                DeleteObject(hbr);
            }
            k = k + 1;
            if (k > 220) k = 1;
        }
        x1 = x1 + x3 - xpixels / 100;
        x2 = x2 + x3 + xpixels / 100;
        y1 = y1 + y3 - ypixels / 100;
        y2 = y2 + y3 + ypixels / 100;
        if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
        {
            escPressed = TRUE;
            return FALSE;
        }
    }
    return TRUE;
    
} // lastTest


static BOOL EndRoutine(HDC hdc)
{
    int i, j, k;
    int x1, x2, x3, x4, y1, y2, y3, y4, i1, i2;
    char paramtext[500];


    if (GetAsyncKeyState(VK_ESCAPE) != keyPress)
    {
        runit = 9;
        escPressed = TRUE;
        return FALSE;
    }

    // ********** Fill screen white/grey *******
        
    hbr = CreateSolidBrush(PALETTEINDEX(7));  
    SelectObject(hdc, hbr);
    Rectangle(hdc, recwin.left, recwin.top, recwin.right, recwin.bottom);
    DeleteObject(hbr);

    // ************ Picture Frames *********

    j = 12;
    for (i=0; i<j; i=i+1)
    {
        hpen = CreatePen(PS_SOLID, 0, PALETTEINDEX(180+i*4));
        hpenOld = SelectObject(hdc, hpen);
        Rectangle(hdc, rect0.left-j+i, rect0.top-j+i, rect0.right+j-i, rect0.bottom+j-i);
        Rectangle(hdc, rect1.left-j+i, rect1.top-j+i, rect1.right+j-i, rect1.bottom+j-i);
        Rectangle(hdc, rect2.left-j+i, rect2.top-j+i, rect2.right+j-i, rect2.bottom+j-i);
        Rectangle(hdc, rect3.left-j+i, rect3.top-j+i, rect3.right+j-i, rect3.bottom+j-i);
        Rectangle(hdc, rect4.left-j+i, rect4.top-j+i, rect4.right+j-i, rect4.bottom+j-i);
        Rectangle(hdc, rect5.left-j+i, rect5.top-j+i, rect5.right+j-i, rect5.bottom+j-i);
        Rectangle(hdc, rect6.left-j+i, rect6.top-j+i, rect6.right+j-i, rect6.bottom+j-i);
        SelectObject(hdc, hpenOld);
        DeleteObject(hpen);
    }

    if (runit == 1)
    {
        hbr = CreateSolidBrush(PALETTEINDEX(0)); 
        SelectObject(hdc, hbr);
        Rectangle(hdc, rect4.left, rect4.top, rect4.right, rect4.bottom);            
        Rectangle(hdc, rect5.left, rect5.top, rect5.right, rect5.bottom);            
        DeleteObject(hbr);
    }
    
    // **************** Screen size & Parameters *****************
   
    ExtTextOut(hdc, incr, ym / 2,    /* x and y coordinates       */
    ETO_CLIPPED,                        /* clips text to rectangle   */
    &rectA,                                /* address of RECT structure */
    screendata,                            /* string to write           */
    whatsize,                           /* characters in string      */
    (LPINT) NULL);                      /* default character spacing */
    sprintf(paramtext, "Log %s, Test seconds %d, Auto Save. Hold down ESC to kill on running. "
                                , logfile, looptime);
    thissize = strlen(paramtext);

    SetBkColor(hdc, PALETTEINDEX(7));
    SetTextColor(hdc, PALETTEINDEX(1)); 
    ExtTextOut(hdc, xm, ym/2-25, ETO_CLIPPED, &recwin, paramtext, thissize, (LPINT) NULL);
    SetBkColor(hdc, PALETTEINDEX(15));
    SetTextColor(hdc, PALETTEINDEX(0)); 

    if (runit > 0)
    {
        sprintf (passtext, " %5.1f seconds                         ", thistime);
        thissize = 24;
        ExtTextOut(hdc, xm, ym / 2, ETO_CLIPPED, &recwin, passtext, thissize, (LPINT) NULL);         
    }

    // **************** All colors middle ***********
    
    x1 = rect6.left;
    y1 = rect6.top;
    x2 = rect6.right;
    y2 = rect6.bottom;
    BitBlt(hdc, x1, y1, (x2-x1), (y2-y1), hdcDest3, 0, 0, SRCCOPY);

    if (runit > 1)
    {
         hbr = CreateSolidBrush(PALETTEINDEX(3)); 
         SelectObject(hdc, hbr);
         Rectangle(hdc, rect0.left, rect0.top, rect0.right, rect0.bottom);
         Rectangle(hdc, rect1.left, rect1.top, rect1.right, rect1.bottom);
         Rectangle(hdc, rect2.left, rect2.top, rect2.right, rect2.bottom);
         Rectangle(hdc, rect3.left, rect3.top, rect3.right, rect3.bottom);            
         Rectangle(hdc, rect5.left, rect5.top, rect5.right, rect5.bottom);            
         DeleteObject(hbr);

         hbr = CreateSolidBrush(PALETTEINDEX(7)); 
         SelectObject(hdc, hbr);
         Rectangle(hdc, rect4.left, rect4.top, rect4.right, rect4.bottom);
         DeleteObject(hbr);
         
         thissize = 24;
         x3 = rect0.right - rect0.left + 1;
         ExtTextOut(hdc, rect0.left-12, rect0.bottom+15, ETO_CLIPPED, &recwin, test1res, thissize, (LPINT) NULL);
         BitBlt(hdc, rect0.left + 1, rect0.top,
                     rect0.right - rect0.left + 1, rect0.bottom - rect0.top, hdcDest, 0, 0, SRCCOPY);

    }
    if (runit > 2)
    {
       i = 0;
       for (j=0; j<incq; j++)   // incq = 10
       {
            for (k=0; k<8; k++)
            {
                 hbr = CreateSolidBrush(PALETTEINDEX(randcol[i+k+j*8])); 
                 SelectObject(hdc, hbr);
                 Ellipse(hdc,  x[j], y[k], x[j] + incp, y[k] + incp);
                 DeleteObject(hbr);
            }
       }   
       thissize = 24;
       ExtTextOut(hdc, rect4.left-12, rect4.top-30, ETO_CLIPPED, &recwin, test2res, thissize, (LPINT) NULL);
    }
    if (runit > 3)
    {
         hbr = CreateSolidBrush(PALETTEINDEX(12)); 
         SelectObject(hdc, hbr);
         Rectangle(hdc, rect1.left, rect1.top, rect1.right, rect1.bottom);
         DeleteObject(hbr);                            
         thissize = 24;
         ExtTextOut(hdc, rect1.left-12, rect1.top-30, ETO_CLIPPED, &recwin, test3res, thissize, (LPINT) NULL);
    }
    if (runit > 4)
    {
         BitBlt(hdc, rect2.left + 1, rect2.top,
                     rect2.right - rect2.left + 1, rect2.bottom - rect2.top, hdcDest2, 0, 0, SRCCOPY);
         hbr = CreateSolidBrush(PALETTEINDEX(0)); 
         SelectObject(hdc, hbr);
         Rectangle(hdc, rect3.left, rect3.top, rect3.right, rect3.bottom);
         DeleteObject(hbr);                            
         thissize = 25;
         ExtTextOut(hdc, rect2.left-12, rect2.bottom+15, ETO_CLIPPED, &recwin, test4res, thissize, (LPINT) NULL);
    }
    if (runit > 5)
    {
          x1 = rect3.left;
          x2 = rect3.right;
          y1 = rect3.top;
          y2 = rect3.bottom;
          i1 = 8;
          if (xpixels >  800) i1 = 16;
          if (xpixels == 800) i1 = 12;
          i2 = i1/2;
          x3 = x2 - x1 - i1;
          x4 = x1 + i2;
          y3 = y2 - y1 - i1;
          y4 = y1 + i2;
          MoveToEx(hdc, x4, y4, NULL);
          hpen = CreatePen(PS_SOLID, 0, PALETTEINDEX(10));
          hpenOld = SelectObject(hdc, hpen);
          for (i=0; i < 9; i++)
          {                  
               LineTo(hdc, x4+x3, y4);
               LineTo(hdc, x4+x3, y4+y3);
               LineTo(hdc, x4,    y4+y3);
               LineTo(hdc, x4,    y4+i2);
               x3 = x3 - i1;
               y3 = y3 - i1;
               x4 = x4 + i2;
               y4 = y4 + i2;                       
          }
          SelectObject(hdc, hpenOld);
          DeleteObject(hpen);
          thissize = 24;
          ExtTextOut(hdc, rect3.left-12, rect3.top-30, ETO_CLIPPED, &recwin, test5res, thissize, (LPINT) NULL);
    }
    if (runit > 6)
    {
         x1 = rect0.left;
         x2 = rect0.right;
         y1 = rect0.top;
         y2 = rect0.bottom;
         x3 = x2 - x1 - 1;

         BitBlt(hdc, rect5.left, rect5.top, (x2-x1), (y2-y1), hdc, rect0.left, rect0.top, SRCCOPY);
         thissize = 24;
         ExtTextOut(hdc, rect5.left-12, rect5.bottom+15, ETO_CLIPPED, &recwin, test6res, thissize, (LPINT) NULL);         
         ExtTextOut(hdc, xm, ym/2, ETO_CLIPPED, &recwin, test7res, thissize, (LPINT) NULL);
    }
    return TRUE;
         
} // EndRoutine
 
static BOOL saveData()
{
   if (escPressed)
   {
       fprintf(outfile, "\n ESCAPE Pressed\n\n");
   }

   if (saveResults)
   {            
       fprintf(outfile, "%5d %4d %2d %8.2f %8.3f %8.2f %8.3f %8.2f %8.2f %8.2f %8.2f\n",
                            xpixels, ypixels, bpixels,
                            speed[0], speed[1], speed[2], speed[3], speed[4], speed[5], speed[6], speed[7]);
       if (gotPixel != 0) fprintf(outfile, "               ERROR 0X%8.8X GetPixel SetPixel was 0X00000000\n", gotPixel);    
   }    
   if (saveEnd)
   {
       gettimedate();
       WhatConfig();
       fprintf(outfile, "\n %s\n", configData1);
       fprintf(outfile, " %s\n", configData2);
       fprintf(outfile, " %s\n", configData3);    
       fprintf (outfile, "\n  Run Time Each Test %d seconds, end at %s \n", looptime, timedate);
   }
   fflush (outfile);
   fclose(outfile);
   return TRUE;

} // saveData


